/*****************************************************************************/
/* XMangekyou - a kaleidoscope for X                                         */
/* Programed by Sakai Hiroaki.                                               */
/*                                                                           */
/* Copyright (c) 1998 Sakai Hiroaki.                                         */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* XMangekyou is a kaleidoscope. You can see many, many, many patterns,      */
/* but, care for yourself from 'trip' filling!                               */
/*                                                                           */
/* XMangekyou is a free software. You may copy or distribute the original    */
/* XMangekyou freely. But, you may not modify or distribute the original     */
/* XMangekyou without permission of the author.                              */
/*****************************************************************************/

/*****************************************************************************/
/* Include Header Files.                                                     */
/*****************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<time.h>
#include<math.h>
#include<X11/Xlib.h>
#include<X11/Intrinsic.h>
#include<X11/StringDefs.h>
#include<X11/keysym.h>
#include<X11/xpm.h>

#include"configure.h"
#include"xm_icon.xpm"

/*****************************************************************************/
/* Parameters.                                                               */
/*****************************************************************************/

#define ON  1
#define OFF 0
#define NO_ANIMATION_FIX -1
#define ANIMATION_SPEED (1.0 / 80.0)
#define DEFAULT_ANIMATION_DATA_FILENAME "animation.dat"
#define SQRT_2 1.414213562373095
#define SQRT_3 1.732050807568877
#define XMANGEKYOU_ICON xm_icon

/*****************************************************************************/
/* Default Values.                                                           */
/*****************************************************************************/

#define DEFAULT_WIDTH  400
#define DEFAULT_HEIGHT 400
#define DEFAULT_ROOT_FLAG OFF
#define DEFAULT_LINES_NUMBER 50
#define DEFAULT_CIRCLES_NUMBER 20
#define DEFAULT_LINE_THICKNESS 5
#define DEFAULT_CIRCLE_SIZE 10
#define DEFAULT_SIZE 80
#define DEFAULT_SPEED 128
#define DEFAULT_CHANGE 1000
#define DEFAULT_WAIT 0

/*****************************************************************************/
/* Structures for XMangekyou.                                                */
/*****************************************************************************/

typedef struct DoublePoint {
  double x;
  double y;
} DoublePoint;

typedef struct CharPoint {
  signed char x;
  signed char y;
} CharPoint;

typedef struct Line {
  DoublePoint point_1;
  DoublePoint point_2;
  GC gc;
} Line;

typedef struct Circle {
  DoublePoint center;
  int r;
  GC gc;
} Circle;

typedef struct PictureLine {
  CharPoint point_1;
  CharPoint point_2;
} PictureLine;

typedef struct PictureCircle {
  CharPoint center;
} PictureCircle;

typedef struct SixPoints {
  DoublePoint point[6];
} SixPoints;

typedef struct Picture {
  unsigned long int count;
  int lines_number;
  int circles_number;
  PictureLine * lines;
  PictureCircle * circles;
} Picture;

typedef struct Animation {
  int pictures_number;
  Picture * pictures;
} Animation;

typedef struct SourcePixmap {
  Pixmap pixmap;
  XPoint center;
  XPoint width;
  int size;
} SourcePixmap;

/*****************************************************************************/
/* Static Data.                                                              */
/*****************************************************************************/

#define COLORS_NUMBER 18
static char * (color_name[COLORS_NUMBER]) = {
"black", "white", "blue", "red", "green", "yellow", "orange",
"pink", "magenta", "cyan", "brown", "gray", "SkyBlue", "GreenYellow",
"purple", "SeaGreen", "gold", "plum"};

#define CLEAR_POINTS_NUMBER 14
static DoublePoint clear_points[CLEAR_POINTS_NUMBER] = {
  { 0.0,-1.0},              {-0.5*SQRT_3, -0.5},     {-0.5*SQRT_3,  0.5},
  { 0.0, 1.0},              { 0.5*SQRT_3,  0.5},     { 0.5*SQRT_3, -0.5},
  { 0.0,-1.0},
  { 0.0,-2.0*SQRT_2/SQRT_3},{-SQRT_2,-SQRT_2/SQRT_3},{-SQRT_2, SQRT_2/SQRT_3},
  { 0.0, 2.0*SQRT_2/SQRT_3},{ SQRT_2, SQRT_2/SQRT_3},{ SQRT_2,-SQRT_2/SQRT_3},
  { 0.0,-2.0*SQRT_2/SQRT_3}
};

#define OFFSET 1.4
static DoublePoint clear_points_offset[CLEAR_POINTS_NUMBER] = {
  { 0.0, -OFFSET}, 
  {-0.5*SQRT_3*OFFSET, -0.5*OFFSET}, {-0.5*SQRT_3*OFFSET, 0.5*OFFSET},
  { 0.0,  OFFSET},
  { 0.5*SQRT_3*OFFSET,  0.5*OFFSET}, { 0.5*SQRT_3*OFFSET,-0.5*OFFSET},
  { 0.0, -OFFSET},
  { 0.0, -OFFSET},
  {-0.5*SQRT_3*OFFSET, -0.5*OFFSET}, {-0.5*SQRT_3*OFFSET, 0.5*OFFSET},
  { 0.0,  OFFSET},
  { 0.5*SQRT_3*OFFSET,  0.5*OFFSET}, { 0.5*SQRT_3*OFFSET,-0.5*OFFSET},
  { 0.0, -OFFSET}
};

/*****************************************************************************/
/* XMangekyou Structure.                                                     */
/*****************************************************************************/

typedef struct struct_xmangekyou {
  XtAppContext app_context;
  Display * display;
  int screen;
  Colormap colormap;
  Widget toplevel;
  Window toplevel_window, root_window, canvas_window;
  Atom atom_1, atom_2;
  unsigned long int foreground_color;
  unsigned long int background_color;
  GC gc_source_clear, gc_source_hex_clear, gc_canvas_clear;
  GC gc_copy_source_canvas, gc_copy_canvas_canvas;
  int icon_width, icon_height;
  Pixmap icon_pixmap;

  int x, y, width, height, depth;
  int root_flag;
  int lines_number, circles_number, line_thickness, circle_size;
  int size, speed, change, wait;
  int degree;
  Line * lines;
  Circle * circles;

  SourcePixmap * source_pixmaps;
  DoublePoint clear_points_rotated[CLEAR_POINTS_NUMBER];
  DoublePoint clear_points_offset_rotated[CLEAR_POINTS_NUMBER];
  XPoint clear_points_rotated_int[CLEAR_POINTS_NUMBER];
  int copy_max, copy_end, copy_number;
  DoublePoint * copy_points;
  DoublePoint * copy_points_rotated;

  char * animation_data_filename;
  int animations_number;
  int animation_flag;
  Animation * animations;
  unsigned long int animation_picture_start_time;
  int animation_type, animation_picture;
  int animation_fix_number;
} XMangekyou;

/*****************************************************************************/
/* Help String.                                                              */
/*****************************************************************************/

static char * help_string =
"\n"
"XMangekyou - a kaleidoscope for X\n"
"\n"
"XMangekyou is a kaleidoscope. You can see many, many, many patterns,\n"
"but, care for yourself from 'trip' filling!\n"
"\n"
"You can use all of the standard X-Toolkit options.\n"
"(x) is default value.\n"
"-display [displayname]\n"
"                This option specifies the name of the X server to use.\n"
"-geometry [geometry]\n"
"                This option specifies the placement of the window\n"
"                on the screen.\n"
"-background [color], -bg [color]\n"
"                This option specifies the background color.\n"
"-h, -help       Output this message.\n"
"-root           Draw a mangekyou on the root window.\n"
"-noroot         Open a new window and draw a mangekyou on it. (Default)\n"
"-lines [x]      This option specifies the number of lines to draw.\n"
"                (Default value is 50)\n"
"-circles [x]    This option specifies the number of circles to draw.\n"
"                (Default value is 20)\n"
"-line-thickness [x]\n"
"                This option specifies the maximum thickness of lines.\n"
"                (Default value is 5)\n"
"-circle-size [x]\n"
"                This option specifies the maximum size of circles.\n"
"                (Default value is 10)\n"
"-size [x]       This option specifies the size of the center of\n"
"                a mangekyou. (80)\n"
"-speed [x]      This option specifies the speed of the rotation. (128)\n"
"-change [x]     This option specifies the speed of the change. (1000)\n"
"-wait [x]       This option specifies the wait. (0)\n"
"-animation-file [filename]\n"
"                This option specifies the filename for the animation.\n"
"                (animation.dat)\n"
"\n"
"Programed by Sakai Hiroaki.\n"
"E-Mail:sakai@miya.ee.kagu.sut.ac.jp\n"
"Thank Tanaka Makoto, Watanabe Takahiro, Umehara Taro, and Masuda Keiko\n"
"for drawing many animations.\n"
"\n"
"Copyright (c) 1998 Sakai Hiroaki.\n"
"\n"
"XMangekyou is a free software. You may copy or distribute the original\n"
"XMangekyou freely. But, you may not modify or distribute the original\n"
"XMangekyou without permission of the author.\n"
"\n"
;

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
